@@ -1,7 +1,7 @@
 #    builder.py -- Classes for building packages
 #    Copyright (C) 2006, 2007 James Westby <jw+debian@jameswestby.net>
 #    
-#    This file is part of bzr-builddeb.
+#    This file is part of breezy-debian.
 #
 #    bzr-builddeb is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -86,7 +86,7 @@ class DebBuild(object):
         format_path = os.path.join(self.target_dir, "debian/source/format")
         if not os.path.isfile(format_path):
             return
-        with file(format_path, 'r') as f:
+        with open(format_path, 'r') as f:
             if f.read().strip() == FORMAT_3_0_QUILT:
                 quilt_push_all(os.path.abspath(self.target_dir))
 
