@@ -7,60 +7,46 @@ from __future__ import absolute_import
 
 import errno
 import os
-from StringIO import StringIO
-import stat
+from io import BytesIO
 import tarfile
-import zipfile
 
-from ... import generate_ids
+from ...upstream_import import (
+    add_implied_parents,
+    common_directory,
+    do_directory,
+    names_of_files,
+    DirWrapper,
+    ZipFileWrapper,
+    )
+try:
+    from ...bzr import generate_ids
+except ImportError:  # brz < 3.0.0
+    from ... import generate_ids
+from ... import urlutils
 from ...controldir import ControlDir
 from ...errors import NoSuchFile, BzrCommandError, NotBranchError
-from ...osutils import (pathjoin, isdir, file_iterator, basename,
-                        file_kind, splitpath, normpath, walkdirs)
+from ...osutils import (
+    file_iterator,
+    basename,
+    file_kind,
+    splitpath,
+    normpath,
+    is_inside_any,
+    )
+from ...sixish import StringIO
 from ...trace import warning
 from ...transform import TreeTransform, resolve_conflicts, cook_conflicts
+from ...transport import get_transport
 from ...workingtree import WorkingTree
-from .bzrtools_bzrtools import open_from_url
 from .errors import UnknownType
 
-class ZipFileWrapper(object):
 
-    def __init__(self, fileobj, mode):
-        self.zipfile = zipfile.ZipFile(fileobj, mode)
-
-    def getmembers(self):
-        for info in self.zipfile.infolist():
-            yield ZipInfoWrapper(self.zipfile, info)
-
-    def extractfile(self, infowrapper):
-        return StringIO(self.zipfile.read(infowrapper.name))
-
-    def add(self, filename):
-        if isdir(filename):
-            self.zipfile.writestr(filename+'/', '')
-        else:
-            self.zipfile.write(filename)
-
-    def close(self):
-        self.zipfile.close()
-
-
-class ZipInfoWrapper(object):
-
-    def __init__(self, zipfile, info):
-        self.info = info
-        self.type = None
-        self.name = info.filename
-        self.zipfile = zipfile
-        self.mode = 0666
-
-    def isdir(self):
-        # Really? Eeeew!
-        return bool(self.name.endswith('/'))
-
-    def isreg(self):
-        # Really? Eeeew!
-        return not self.isdir()
+def open_from_url(location):
+    location = urlutils.normalize_url(location)
+    dirname, basename = urlutils.split(location)
+    if location.endswith('/') and not basename.endswith('/'):
+        basename += '/'
+    return get_transport(dirname).get(basename)
 
 
 files_to_ignore = set(
@@ -68,99 +54,6 @@ files_to_ignore = set(
      '.bzr-builddeb'])
 
 
-class DirWrapper(object):
-    def __init__(self, fileobj, mode='r'):
-        assert mode == 'r', mode
-        self.root = os.path.realpath(fileobj.read())
-
-    def __repr__(self):
-        return 'DirWrapper(%r)' % self.root
-
-    def getmembers(self):
-        for _, dir_block in walkdirs(self.root):
-            for relpath, _, _, stat_result, _ in dir_block:
-                yield FileInfo(self.root, relpath, stat_result)
-
-    def extractfile(self, member):
-        return open(member.fullpath)
-
-
-class FileInfo(object):
-
-    def __init__(self, root, filepath, stat):
-        self.fullpath = pathjoin(root, filepath)
-        self.root = root
-        if filepath != '':
-            self.name = pathjoin(basename(root), filepath)
-        else:
-            self.name = basename(root)
-        self.type = None
-        self.mode = stat.st_mode
-        if self.isdir():
-            self.name += '/'
-
-    def __repr__(self):
-        return 'FileInfo(%r)' % self.name
-
-    def isreg(self):
-        return stat.S_ISREG(self.mode)
-
-    def isdir(self):
-        return stat.S_ISDIR(self.mode)
-
-    def issym(self):
-        if stat.S_ISLNK(self.mode):
-            self.linkname = os.readlink(self.fullpath)
-            return True
-        else:
-            return False
-
-
-def top_path(path):
-    """Return the top directory given in a path."""
-    components = splitpath(normpath(path))
-    if len(components) > 0:
-        return components[0]
-    else:
-        return ''
-
-
-def common_directory(names):
-    """Determine a single directory prefix from a list of names"""
-    prefixes = set()
-    prefixes.update(map(top_path, names))
-    if '' in prefixes:
-        prefixes.remove('')
-    if len(prefixes) != 1:
-        return None
-    prefix = prefixes.pop()
-    if prefix == '':
-        return None
-    return prefix
-
-
-def do_directory(tt, trans_id, tree, relative_path, path):
-    if isdir(path) and tree.is_versioned(relative_path):
-        tt.cancel_deletion(trans_id)
-    else:
-        tt.create_directory(trans_id)
-
-
-def add_implied_parents(implied_parents, path):
-    """Update the set of implied parents from a path"""
-    parent = os.path.dirname(path)
-    if parent in implied_parents:
-        return
-    implied_parents.add(parent)
-    add_implied_parents(implied_parents, parent)
-
-
-def names_of_files(tar_file):
-    for member in tar_file.getmembers():
-        if member.type != "g":
-            yield member.name
-
-
 def should_ignore(relative_path):
     parts = splitpath(relative_path)
     if not parts:
@@ -180,30 +73,38 @@ def import_tar(tree, tar_input, file_ids_from=None, target_tree=None):
     import_archive(tree, tar_file, file_ids_from=file_ids_from,
             target_tree=target_tree)
 
+
 def import_zip(tree, zip_input, file_ids_from=None, target_tree=None):
     zip_file = ZipFileWrapper(zip_input, 'r')
     import_archive(tree, zip_file, file_ids_from=file_ids_from,
             target_tree=target_tree)
 
-def import_dir(tree, dir, file_ids_from=None, target_tree=None):
-    dir_input = StringIO(dir)
-    dir_file = DirWrapper(dir_input)
-    import_archive(tree, dir_file, file_ids_from=file_ids_from,
-            target_tree=target_tree)
 
-def import_archive(tree, archive_file, file_ids_from=None, target_tree=None):
+def import_dir(tree, dir, file_ids_from=None, target_tree=None, exclude=None):
+    dir_input = BytesIO(dir.encode('utf-8'))
+    dir_input.seek(0)
+    dir_file = DirWrapper(dir_input)
+    import_archive(
+        tree, dir_file, file_ids_from=file_ids_from, target_tree=target_tree,
+        exclude=exclude)
+
+
+def import_archive(tree, archive_file, file_ids_from=None, target_tree=None,
+                   exclude=None):
     if file_ids_from is None:
         file_ids_from = []
     for other_tree in file_ids_from:
         other_tree.lock_read()
     try:
-        return _import_archive(tree, archive_file, file_ids_from,
-                target_tree=target_tree)
+        return _import_archive(
+            tree, archive_file, file_ids_from, target_tree=target_tree,
+            exclude=exclude)
     finally:
         for other_tree in file_ids_from:
             other_tree.unlock()
 
-def _get_paths_to_process(archive_file, prefix, implied_parents):
+
+def _get_paths_to_process(archive_file, prefix, implied_parents, exclude=None):
     to_process = set()
     for member in archive_file.getmembers():
         if member.type == 'g':
@@ -219,12 +120,15 @@ def _get_paths_to_process(archive_file, prefix, implied_parents):
             continue
         if should_ignore(relative_path):
             continue
+        if exclude and is_inside_any(exclude, relative_path):
+            continue
         add_implied_parents(implied_parents, relative_path)
         to_process.add((relative_path, member))
     return to_process
 
 
-def _import_archive(tree, archive_file, file_ids_from, target_tree=None):
+def _import_archive(tree, archive_file, file_ids_from, target_tree=None,
+                    exclude=None):
     prefix = common_directory(names_of_files(archive_file))
     tt = TreeTransform(tree)
     try:
@@ -240,7 +144,7 @@ def _import_archive(tree, archive_file, file_ids_from, target_tree=None):
         implied_parents = set()
         seen = set()
         to_process = _get_paths_to_process(archive_file, prefix,
-                implied_parents)
+                implied_parents, exclude=exclude)
         renames = {}
 
         # First we find the renames
@@ -315,7 +219,7 @@ def _import_archive(tree, archive_file, file_ids_from, target_tree=None):
             if member.isreg():
                 tt.create_file(file_iterator(archive_file.extractfile(member)),
                                trans_id)
-                executable = (member.mode & 0111) != 0
+                executable = (member.mode & 0o111) != 0
                 tt.set_executability(executable, trans_id)
             elif member.isdir():
                 do_directory(tt, trans_id, tree, relative_path, path)
@@ -350,7 +254,7 @@ def _import_archive(tree, archive_file, file_ids_from, target_tree=None):
             tt.unversion_file(tt.trans_id_tree_path(path))
 
         for conflict in cook_conflicts(resolve_conflicts(tt), tt):
-            warning(conflict)
+            warning('%s', conflict)
         tt.apply()
     finally:
         tt.finalize()
@@ -377,8 +281,8 @@ def do_import(source, tree_directory=None):
             try:
                 tar_input = open_from_url(source)
                 if source.endswith('.bz2'):
-                    tar_input = StringIO(tar_input.read().decode('bz2'))
-            except IOError, e:
+                    tar_input = BytesIO(tar_input.read().decode('bz2'))
+            except IOError as e:
                 if e.errno == errno.ENOENT:
                     raise NoSuchFile(source)
             try:
@@ -388,8 +292,6 @@ def do_import(source, tree_directory=None):
         elif source.endswith('.zip'):
             import_zip(tree, open_from_url(source))
         elif file_kind(source) == 'directory':
-            s = StringIO(source)
-            s.seek(0)
-            import_dir(tree, s)
+            import_dir(tree, source)
         else:
             raise BzrCommandError('Unhandled import source')
