@@ -4,7 +4,7 @@
 #                  2007 Reinhard Tartler <siretart@tauware.de>
 #                  2008-2011 Canonical Ltd.
 #
-#    This file is part of bzr-builddeb.
+#    This file is part of breezy-debian.
 #
 #    bzr-builddeb is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -37,10 +37,15 @@ from ...errors import (
     BzrCommandError,
     FileExists,
     NotBranchError,
+    NoColocatedBranchSupport,
+    NotLocalUrl,
     NoWorkingTree,
+    UnsupportedOperation,
     )
 from ...option import Option
+from ...sixish import text_type
 from ...trace import note, warning
+from ...transport import get_transport
 from ...workingtree import WorkingTree
 
 from . import (
@@ -80,12 +85,42 @@ native_opt = Option('native',
     help="Build a native package.")
 export_upstream_opt = Option('export-upstream',
     help="Create the .orig.tar.gz from specified bzr branch before building.",
-    type=unicode, argname="BRANCH")
+    type=text_type, argname="BRANCH")
 export_upstream_revision_opt = Option('export-upstream-revision',
     help="Select the upstream revision that will be exported.",
     type=str, argname="REVISION")
 
 
+def changes_filename(package, version, arch):
+    non_epoch_version = version.upstream_version
+    if version.debian_version is not None:
+        non_epoch_version += "-%s" % version.debian_version
+    return "%s_%s_%s.changes" % (package,
+            non_epoch_version, arch)
+
+
+def _check_tree(tree, strict=False):
+    if strict:
+        for unknown in tree.unknowns():
+            from .errors import StrictBuildFailed
+            raise StrictBuildFailed()
+
+    if len(tree.conflicts()) > 0:
+        raise BzrCommandError(
+            "There are conflicts in the working tree. "
+            "You must resolve these before building.")
+
+
+def _get_build_architecture():
+    import subprocess
+    try:
+        return subprocess.check_output(
+            ['dpkg-architecture', '-qDEB_BUILD_ARCH']).strip()
+    except subprocess.CalledProcessError as e:
+        raise BzrCommandError(
+            "Could not find the build architecture: %s" % e)
+
+
 def _get_changelog_info(tree, last_version=None, package=None, distribution=None):
     from .util import (
         find_changelog,
@@ -129,6 +164,94 @@ def _get_changelog_info(tree, last_version=None, package=None, distribution=None
             changelog, top_level)
 
 
+def _get_upstream_branch_source(export_upstream, export_upstream_revision,
+        config, version):
+    if export_upstream is None and config.upstream_branch:
+        export_upstream = config.upstream_branch
+    if export_upstream is None:
+        return None
+    from .upstream.branch import (
+        LazyUpstreamBranchSource,
+        )
+    upstream_revision_map = {}
+    if export_upstream_revision:
+        upstream_revision_map[version] = export_upstream_revision
+    upstream_source = LazyUpstreamBranchSource(
+        export_upstream, config=config, upstream_revision_map=upstream_revision_map)
+    return upstream_source
+
+
+def _get_upstream_sources(local_tree, branch, build_type, config,
+                          upstream_version, top_level, export_upstream=None,
+                          export_upstream_revision=None):
+    from .upstream import (
+        AptSource,
+        SelfSplitSource,
+        UScanSource,
+        )
+    from .upstream.pristinetar import (
+        PristineTarSource,
+        )
+    from .upstream.branch import (
+        LazyUpstreamBranchSource,
+        )
+    yield PristineTarSource(branch)
+    yield AptSource()
+    if build_type == BUILD_TYPE_MERGE:
+        upstream_branch_source = _get_upstream_branch_source(
+            export_upstream, export_upstream_revision, config,
+            upstream_version)
+        if upstream_branch_source:
+            yield upstream_branch_source
+    elif config.upstream_branch is not None:
+        yield LazyUpstreamBranchSource(config.upstream_branch)
+    yield UScanSource(local_tree, top_level)
+
+    if build_type == BUILD_TYPE_SPLIT:
+        yield SelfSplitSource(local_tree)
+
+
+def _get_distiller(tree, branch, changelog, build_type, config,
+                   contains_upstream_source=True, top_level=False,
+                   orig_dir=default_orig_dir, use_existing=False,
+                   export_upstream=None, export_upstream_revision=None):
+    from .util import (
+        guess_build_type,
+        )
+    from .upstream import (
+        UpstreamProvider,
+        )
+    from .source_distiller import (
+        FullSourceDistiller,
+        MergeModeDistiller,
+        NativeSourceDistiller,
+        )
+
+    if build_type is None:
+        build_type = config.build_type
+    if build_type is None:
+        build_type = guess_build_type(tree, changelog.version,
+            contains_upstream_source)
+
+    note(gettext("Building package in %s mode") % build_type)
+
+    upstream_sources = list(_get_upstream_sources(
+        tree, branch, build_type=build_type, config=config,
+        upstream_version=changelog.version.upstream_version,
+        top_level=top_level, export_upstream=export_upstream,
+        export_upstream_revision=export_upstream_revision))
+
+    upstream_provider = UpstreamProvider(changelog.package,
+        changelog.version.upstream_version, orig_dir, upstream_sources)
+
+    if build_type == BUILD_TYPE_MERGE:
+        return MergeModeDistiller(tree, upstream_provider,
+                top_level=top_level, use_existing=use_existing)
+    elif build_type == BUILD_TYPE_NATIVE:
+        return NativeSourceDistiller(tree)
+    else:
+        return FullSourceDistiller(tree, upstream_provider)
+
 
 class cmd_builddeb(Command):
     """Builds a Debian package from a branch.
@@ -203,11 +326,16 @@ class cmd_builddeb(Command):
         package_merge_opt]
 
     def _get_tree_and_branch(self, location):
-        import urlparse
         if location is None:
             location = "."
-        is_local = urlparse.urlsplit(location)[0] in ('', 'file')
-        controldir, relpath = ControlDir.open_containing(location)
+        transport = get_transport(location)
+        try:
+            transport.local_abspath('.')
+        except NotLocalUrl:
+            is_local = False
+        else:
+            is_local = True
+        controldir, relpath = ControlDir.open_containing_from_transport(transport)
         tree, branch = controldir._get_tree_branch()
         return tree, branch, is_local, controldir.user_url
 
@@ -293,17 +421,6 @@ class cmd_builddeb(Command):
             source = True
         return branch, build_options, source
 
-    def _get_upstream_branch(self, export_upstream, export_upstream_revision,
-            config, version):
-        from .upstream.branch import (
-            LazyUpstreamBranchSource,
-            )
-        upstream_source = LazyUpstreamBranchSource(export_upstream,
-            config=config)
-        if export_upstream_revision:
-            upstream_source.upstream_revision_map[version.encode("utf-8")] = export_upstream_revision
-        return upstream_source
-
     def run(self, branch_or_build_options_list=None, verbose=False,
             working_tree=False,
             export_only=False, dont_purge=False, use_existing=False,
@@ -313,30 +430,12 @@ class cmd_builddeb(Command):
             quick=False, reuse=False, native=None,
             source=False, revision=None, package_merge=None,
             strict=False):
-        import commands
-        from .source_distiller import (
-                FullSourceDistiller,
-                MergeModeDistiller,
-                NativeSourceDistiller,
-                )
+        import subprocess
         from .builder import DebBuild
         from .errors import (
             NoPreviousUpload,
-            StrictBuildFailed
             )
         from .hooks import run_hook
-        from .upstream.branch import (
-            LazyUpstreamBranchSource,
-            )
-        from .upstream import (
-            AptSource,
-            SelfSplitSource,
-            UScanSource,
-            UpstreamProvider,
-            )
-        from .upstream.pristinetar import (
-            PristineTarSource,
-            )
         from .util import (
             dget_changes,
             find_changelog,
@@ -348,32 +447,19 @@ class cmd_builddeb(Command):
         location, build_options, source = self._branch_and_build_options(
                 branch_or_build_options_list, source)
         tree, branch, is_local, location = self._get_tree_and_branch(location)
-        tree, working_tree = self._get_build_tree(revision, tree, branch)
-        if strict:
-            for unknown in tree.unknowns():
-                raise StrictBuildFailed()
-
-        if len(tree.conflicts()) > 0:
-            raise BzrCommandError(
-                "There are conflicts in the working tree. "
-                "You must resolve these before building.")
+        tree, is_working_tree = self._get_build_tree(revision, tree, branch)
+        _check_tree(tree, strict=strict)
 
         with tree.lock_read():
-            config = debuild_config(tree, working_tree)
+            config = debuild_config(tree, is_working_tree)
             if reuse:
                 note(gettext("Reusing existing build dir"))
                 dont_purge = True
                 use_existing = True
             build_type = self._build_type(merge, native, split)
-            if build_type is None:
-                build_type = config.build_type
+
             contains_upstream_source = tree_contains_upstream_source(tree)
             (changelog, top_level) = find_changelog(tree, not contains_upstream_source)
-            if build_type is None:
-                build_type = guess_build_type(tree, changelog.version,
-                    contains_upstream_source)
-
-            note(gettext("Building package in %s mode") % build_type)
 
             if package_merge:
                 try:
@@ -394,39 +480,12 @@ class cmd_builddeb(Command):
             result_dir, build_dir, orig_dir = self._get_dirs(config,
                 location or ".", is_local, result_dir, build_dir, orig_dir)
 
-            upstream_sources = [
-                PristineTarSource(tree, branch),
-                AptSource(),
-                ]
-            if build_type == BUILD_TYPE_MERGE:
-                if export_upstream is None and config.upstream_branch:
-                    export_upstream = config.upstream_branch
-                if export_upstream:
-                    upstream_branch_source = self._get_upstream_branch(
-                        export_upstream, export_upstream_revision, config,
-                        changelog.version.upstream_version)
-                    upstream_sources.append(upstream_branch_source)
-            elif not native and config.upstream_branch is not None:
-                upstream_sources.append(LazyUpstreamBranchSource(config.upstream_branch))
-            upstream_sources.extend([
-                UScanSource(tree, top_level),
-                ])
-            if build_type == BUILD_TYPE_SPLIT:
-                upstream_sources.append(SelfSplitSource(tree))
-
-            upstream_provider = UpstreamProvider(changelog.package,
-                changelog.version.upstream_version, orig_dir, upstream_sources)
-
-            if build_type == BUILD_TYPE_MERGE:
-                distiller_cls = MergeModeDistiller
-            elif build_type == BUILD_TYPE_NATIVE:
-                distiller_cls = NativeSourceDistiller
-            else:
-                distiller_cls = FullSourceDistiller
-
-            distiller = distiller_cls(tree, upstream_provider,
-                    top_level=top_level, use_existing=use_existing,
-                    is_working_tree=working_tree)
+            distiller = _get_distiller(
+                tree, branch, build_type=build_type, config=config,
+                changelog=changelog, contains_upstream_source=contains_upstream_source,
+                orig_dir=orig_dir, use_existing=use_existing, top_level=top_level,
+                export_upstream=export_upstream,
+                export_upstream_revision=export_upstream_revision)
 
             build_source_dir = os.path.join(build_dir,
                     "%s-%s" % (changelog.package,
@@ -446,15 +505,8 @@ class cmd_builddeb(Command):
                 if source:
                     arch = "source"
                 else:
-                    status, arch = commands.getstatusoutput(
-                        'dpkg-architecture -qDEB_BUILD_ARCH')
-                    if status > 0:
-                        raise BzrCommandError("Could not find the build architecture")
-                non_epoch_version = changelog.version.upstream_version
-                if changelog.version.debian_version is not None:
-                    non_epoch_version += "-%s" % changelog.version.debian_version
-                changes = "%s_%s_%s.changes" % (changelog.package,
-                        non_epoch_version, arch)
+                    arch = _get_build_architecture()
+                changes = changes_filename(changelog.package, changelog.version, arch)
                 changes_path = os.path.join(build_dir, changes)
                 if not os.path.exists(changes_path):
                     if result_dir is not None:
@@ -478,7 +530,7 @@ class cmd_get_orig_source(Command):
 
     directory_opt = Option('directory',
         help='Directory from which to retrieve the packaging data',
-        short_name='d', type=unicode)
+        short_name='d', type=text_type)
 
     takes_options = [directory_opt]
     takes_args = ["version?"]
@@ -508,7 +560,7 @@ class cmd_get_orig_source(Command):
 
         upstream_provider = UpstreamProvider(changelog.package,
                 str(version), orig_dir,
-                [PristineTarSource(tree, tree.branch),
+                [PristineTarSource(tree.branch),
                  AptSource(),
                  UScanSource(tree, larstiq) ])
 
@@ -578,10 +630,10 @@ class cmd_merge_upstream(Command):
             "this release is targetted at.", type=str)
     directory_opt = Option('directory',
                            help='Working tree into which to merge.',
-                           short_name='d', type=unicode)
+                           short_name='d', type=text_type)
     last_version_opt = Option('last-version',
                               help='The full version of the last time '
-                              'upstream was merged.', type=str)
+                              'upstream was merged.', type=text_type)
     force_opt = Option('force',
                        help=('Force a merge even if the upstream branch '
                              'has not changed.'))
@@ -606,7 +658,7 @@ class cmd_merge_upstream(Command):
         try:
             changelog_add_new_version(tree, version, distribution_name,
                 changelog, package)
-        except DchError, e:
+        except DchError as e:
             note(e)
             raise BzrCommandError('Adding a new changelog stanza after the '
                     'merge had completed failed. Add the new changelog '
@@ -730,26 +782,28 @@ class cmd_merge_upstream(Command):
 
             if upstream_branch is not None:
                 upstream_branch = Branch.open(upstream_branch)
-            elif location is not None:
+            elif location is not None and not os.path.isfile(location):
                 try:
                     upstream_branch = Branch.open(location)
                 except NotBranchError:
                     upstream_branch = None
             elif upstream_branch is None and config.upstream_branch is not None:
+                note(gettext("Using upstream branch %s (from configuration)") % config.upstream_branch)
                 upstream_branch = Branch.open(config.upstream_branch)
             else:
                 upstream_branch = None
 
             if upstream_branch is not None:
-                upstream_branch_source = UpstreamBranchSource(
-                    upstream_branch, config=config)
+                upstream_branch_source = UpstreamBranchSource.from_branch(
+                    upstream_branch, config=config, local_dir=tree.controldir)
             else:
                 upstream_branch_source = None
 
             if location is not None:
                 try:
-                    primary_upstream_source = UpstreamBranchSource(
-                        Branch.open(location), config=config)
+                    primary_upstream_source = UpstreamBranchSource.from_branch(
+                        Branch.open(location), config=config,
+                        local_dir=tree.controldir)
                 except NotBranchError:
                     primary_upstream_source = TarfileSource(location, version)
             else:
@@ -790,7 +844,6 @@ class cmd_merge_upstream(Command):
                 else:
                     raise BzrCommandError(gettext("You must specify the "
                         "version number using --version."))
-            assert isinstance(version, str)
             note(gettext("Using version string %s.") % (version))
             # Look up the revision id from the version string
             if upstream_revisions is None and upstream_branch_source is not None:
@@ -887,7 +940,7 @@ class cmd_import_dsc(Command):
             open_file_via_transport,
             )
         cache = DscCache()
-        files_list.sort(cmp=DscComp(cache).cmp)
+        files_list.sort(key=DscComp(cache).key)
         if not os.path.exists(orig_target):
             os.makedirs(orig_target)
         for dscname in files_list:
@@ -895,11 +948,8 @@ class cmd_import_dsc(Command):
             def get_dsc_part(from_transport, filename):
                 from_f = open_file_via_transport(filename, from_transport)
                 contents = from_f.read()
-                to_f = open(os.path.join(orig_target, filename), 'wb')
-                try:
+                with open(os.path.join(orig_target, filename), 'wb') as to_f:
                     to_f.write(contents)
-                finally:
-                    to_f.close()
             base, filename = urlutils.split(dscname)
             from_transport = cache.get_transport(dscname)
             get_dsc_part(from_transport, filename)
@@ -933,7 +983,7 @@ class cmd_import_dsc(Command):
             if files_list is None:
                 files_list = []
             if file is not None:
-                if isinstance(file, unicode):
+                if isinstance(file, text_type):
                     file = file.encode('utf-8')
                 sources_file = open_file(file)
                 for line in sources_file:
@@ -965,7 +1015,7 @@ class cmd_import_dsc(Command):
                     if not db.pristine_upstream_source.has_version(
                             changelog.package, last_version.upstream_version):
                         raise BzrCommandError(gettext("Unable to find the tag "
-                            "for the previous upstream version, %(version)s, in the "
+                            "for the previous upstream version (%(version)s) in the "
                             "branch. Consider importing it via import-upstream."
                             "If it is already present in the branch please "
                             "make sure it is tagged as %(tag)r.") %
@@ -1032,7 +1082,6 @@ class cmd_import_upstream(Command):
             md5sum_filename,
             )
         # TODO: search for similarity etc.
-        version = version.encode('utf8')
         branch, _ = Branch.open_containing('.')
         if upstream_branch is None:
             upstream = None
@@ -1061,7 +1110,7 @@ class cmd_import_upstream(Command):
                     upstream.repository.fetch(branch.repository, parent)
             db.extract_upstream_tree(base_revisions, tempdir)
             parents = {}
-            for name, base_revid in base_revisions.iteritems():
+            for name, base_revid in base_revisions.items():
                 parents[name] = [base_revid]
         else:
             parents = {}
@@ -1183,12 +1232,12 @@ class cmd_builddeb_do(Command):
 
         upstream_provider = UpstreamProvider(changelog.package,
                 changelog.version.upstream_version, orig_dir,
-                [PristineTarSource(t, t.branch),
+                [PristineTarSource(t.branch),
                  AptSource(),
                  UScanSource(t, top_level) ])
 
-        distiller = MergeModeDistiller(t, upstream_provider,
-                top_level=top_level)
+        distiller = MergeModeDistiller(
+                t, upstream_provider, top_level=top_level)
 
         build_source_dir = os.path.join(build_dir,
                 changelog.package + "-" + changelog.version.upstream_version)
@@ -1239,6 +1288,7 @@ class cmd_mark_uploaded(Command):
             "marked.")
 
     takes_options = [merge_opt, force]
+    hidden = True
 
     def run(self, merge=None, force=None):
         from .import_dsc import (
@@ -1345,7 +1395,7 @@ class cmd_dep3_patch(Command):
 
     directory_opt = Option('directory',
                            help='Packaging tree for which to generate patch.',
-                           short_name='d', type=unicode)
+                           short_name='d', type=text_type)
 
     no_upstream_check_opt = Option('no-upstream-check',
         help="Don't check whether patch has been merged upstream.")
@@ -1391,9 +1441,9 @@ class cmd_dep3_patch(Command):
             # that revisions commits message
             lhs_history = graph.iter_lefthand_ancestry(revision_id,
                 [base_revid])
-            rev = branch.repository.get_revision(lhs_history.next())
+            rev = branch.repository.get_revision(next(lhs_history))
             try:
-                lhs_history.next()
+                next(lhs_history)
             except StopIteration:
                 description = rev.message
         origin = describe_origin(branch, revision_id)
@@ -1413,3 +1463,130 @@ class cmd_dep3_patch(Command):
             revision_id, bugs=bugs, authors=authors, origin=origin,
             forwarded=forwarded, applied_upstream=applied_upstream,
             description=description, last_update=last_update)
+
+
+class LocalTree(object):
+
+    def __init__(self, branch):
+        self.branch = branch
+        self._td = None
+
+    def __enter__(self):
+        try:
+            return self.branch.controldir.open_workingtree()
+        except (NoWorkingTree, NotLocalUrl):
+            pass
+        # Remote, inaccessible
+        self._td = tempfile.mkdtemp()
+        to_dir = self.branch.controldir.sprout(
+                get_transport(self._td).base, None,
+                create_tree_if_local=True,
+                source_branch=self.branch)
+        try:
+            pristine_tar_branch = self.branch.controldir.open_branch(
+                name="pristine-tar")
+        except (NotBranchError, NoColocatedBranchSupport):
+            pass
+        else:
+            pristine_tar_branch.push(to_dir.create_branch("pristine-tar"))
+        return to_dir.open_workingtree()
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if self._td is not None:
+            shutil.rmtree(self._td)
+        return False
+
+
+def _build_helper(local_tree, branch, target_dir, builder):
+    # TODO(jelmer): Integrate this with cmd_builddeb
+    import subprocess
+    from .hooks import run_hook
+    from .builder import (
+        DebBuild,
+        )
+    from .util import (
+        find_changelog,
+        dget_changes,
+        tree_contains_upstream_source,
+        )
+
+    (changelog, top_level) = find_changelog(
+        local_tree, False, max_blocks=2)
+
+    config = debuild_config(local_tree, local_tree)
+    contains_upstream_source = tree_contains_upstream_source(local_tree)
+
+    distiller = _get_distiller(
+            local_tree, branch, build_type=None, config=config,
+            changelog=changelog,
+            contains_upstream_source=contains_upstream_source,
+            top_level=top_level)
+
+    bd = tempfile.mkdtemp()
+    try:
+        build_source_dir = os.path.join(bd,
+                changelog.package + "-" + changelog.version.upstream_version)
+        builder = DebBuild(
+                distiller, build_source_dir,
+                builder,
+                use_existing=False)
+        builder.prepare()
+        run_hook(local_tree, 'pre-export', config)
+        builder.export()
+        run_hook(local_tree, 'pre-build', config, wd=build_source_dir)
+        builder.build()
+        run_hook(local_tree, 'post-build', config, wd=build_source_dir)
+        changes = changes_filename(changelog.package, changelog.version, 'source')
+        changes_path = os.path.join(bd, changes)
+        if target_dir is not None:
+            if not os.path.exists(changes_path):
+                raise BzrCommandError("Could not find the .changes "
+                        "file from the build: %s" % changes_path)
+            return dget_changes(changes_path, target_dir)
+    finally:
+        shutil.rmtree(bd)
+
+
+class cmd_debrelease(Command):
+    """Release a new version of a package.
+
+    This will update the latest distribution in the changelog,
+    build the package and upload.
+
+    Location can be remote.
+    """
+
+    strict_opt = Option('strict',
+               help='Refuse to build if there are unknown files in'
+               ' the working tree, --no-strict disables the check.')
+
+    takes_args = ['location?']
+    takes_options = [strict_opt,
+            Option('skip-upload', help='Skip upload.'), builder_opt]
+
+    def run(self, location='.', strict=True, skip_upload=False,
+            builder="sbuild --source --source-only-changes"):
+        from .release import release
+        from .util import (
+            dput_changes,
+            )
+
+        branch = Branch.open(location)
+        # preserve whatever source format we have.
+        # TODO(jelmer): Use the local tree if there is one, but check it's
+        # clean.
+        with LocalTree(branch) as local_tree:
+            _check_tree(local_tree, strict)
+            release(local_tree)
+
+            td = tempfile.mkdtemp()
+            try:
+                changes_file = _build_helper(
+                        local_tree, local_tree.branch,
+                        target_dir=(td if not skip_upload else None),
+                        builder=builder)
+                if not skip_upload:
+                    dput_changes(changes_file)
+            finally:
+                shutil.rmtree(td)
+            local_tree.branch.push(branch)
