@@ -1,7 +1,7 @@
 #    config.py -- Configuration of bzr-builddeb from files
 #    Copyright (C) 2006 James Westby <jw+debian@jameswestby.net>
 #    
-#    This file is part of bzr-builddeb.
+#    This file is part of breezy-debian.
 #
 #    bzr-builddeb is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -20,11 +20,14 @@
 
 from __future__ import absolute_import
 
+import yaml
+
 from ...config import (
   configobj,
   ConfigObj,
   TreeConfig,
   )
+from ...errors import NoSuchFile
 from ...trace import mutter, warning
 
 
@@ -55,6 +58,31 @@ class SvnBuildPackageMappedConfig(object):
     return None
 
 
+class UpstreamMetadataConfig(object):
+  """Config object that represents debian/upstream/metadata.
+  """
+
+  filename = 'debian/upstream/metadata'
+
+  def __init__(self, text):
+    self.metadata = yaml.safe_load(text)
+
+  def get_value(self, section, option):
+    if section == "BUILDDEB":
+      if option == "upstream-branch":
+        return self.metadata.get('Repository')
+    raise KeyError
+
+  def __getitem__(self, key):
+    return self.get_value(key, "BUILDDEB")
+
+  def get_bool(self, section, option):
+    raise KeyError
+
+  def as_bool(self, option):
+    raise KeyError
+
+
 class DebBuildConfig(object):
   """Holds the configuration settings for builddeb. These are taken from
   a hierarchy of config files. .bzr-builddeb/local.conf then 
@@ -89,22 +117,22 @@ class DebBuildConfig(object):
     ...      (d + 'local.conf', False),
     ...      (d + 'user.conf', True), 
     ...      (d + 'default.conf', False)])
-    >>> print c.orig_dir
+    >>> print(c.orig_dir)
     None
-    >>> print c.merge
+    >>> print(c.merge)
     True
-    >>> print c.build_dir
+    >>> print(c.build_dir)
     defaultbuild
-    >>> print c.result_dir
+    >>> print(c.result_dir)
     userresult
-    >>> print c.builder
+    >>> print(c.builder)
     userbuild
     """
     self._config_files = []
     for input in files:
       try:
         config = ConfigObj(input[0])
-      except configobj.ParseError, e:
+      except configobj.ParseError as e:
         if len(input) > 2:
           content = input[2]
         else:
@@ -129,6 +157,12 @@ class DebBuildConfig(object):
           pass # Not a svn tree
       except ImportError:
         pass # No svn, apparently
+      try:
+        upstream_metadata_text = tree.get_file_text(UpstreamMetadataConfig.filename)
+      except NoSuchFile:
+        pass
+      else:
+        self._config_files.append((UpstreamMetadataConfig(upstream_metadata_text), False))
     self.user_config = None
 
   def set_user_config(self, user_conf):
@@ -235,7 +269,7 @@ class DebBuildConfig(object):
       if not trusted or config_file[1]:
         (found, value) = self._get_bool(config_file[0], key)
         if found:
-          mutter("Using %s for %s, taken from %s", value, key,
+          mutter("Using %s for %s, taken from %s", str(value), key,
                  config_file[0].filename)
           return value
     return default
