@@ -53,6 +53,7 @@ from ...errors import (
     UnrelatedBranches,
     )
 from ...revision import NULL_REVISION
+from ...sixish import text_type, viewitems
 from ...trace import warning, mutter
 from ...transport import (
     get_transport,
@@ -113,6 +114,10 @@ class DscComp(object):
     def __init__(self, cache):
         self.cache = cache
 
+    def key(self, dscname):
+        dsc = self.cache.get_dsc(dscname)
+        return Version(dsc['Version'])
+
     def cmp(self, dscname1, dscname2):
         dsc1 = self.cache.get_dsc(dscname1)
         dsc2 = self.cache.get_dsc(dscname2)
@@ -238,7 +243,7 @@ class DistributionBranch(object):
         self.pristine_upstream_tree = pristine_upstream_tree
         if pristine_upstream_branch is not None:
             self.pristine_upstream_source = PristineTarSource(
-                branch=pristine_upstream_branch, tree=pristine_upstream_tree)
+                branch=pristine_upstream_branch)
         else:
             self.pristine_upstream_source = None
         self.get_lesser_branches = None
@@ -301,6 +306,7 @@ class DistributionBranch(object):
         if md5 is None:
             return True
         rev = branch.repository.get_revision(revid)
+        # TODO(jelmer): Check the commit message for git repositories
         try:
             return rev.properties['deb-md5'] == md5
         except KeyError:
@@ -325,12 +331,13 @@ class DistributionBranch(object):
         tag_name = self.tag_name(version)
         if self._has_version(self.branch, tag_name, md5=md5):
             return True
-        debian_tag_name = "debian-" + tag_name
-        if self._has_version(self.branch, debian_tag_name, md5=md5):
-            return True
-        ubuntu_tag_name = "ubuntu-" + tag_name
-        if self._has_version(self.branch, ubuntu_tag_name, md5=md5):
-            return True
+        # TODO(jelmer): There is some overlap here with revid_of_version
+        for debian_tag_name in ["debian-" + tag_name, "debian/" + tag_name]:
+            if self._has_version(self.branch, debian_tag_name, md5=md5):
+                return True
+        for ubuntu_tag_name in ["ubuntu-" + tag_name, "ubuntu/" + tag_name]:
+            if self._has_version(self.branch, ubuntu_tag_name, md5=md5):
+                return True
         return False
 
     def contained_versions(self, versions):
@@ -412,12 +419,12 @@ class DistributionBranch(object):
         tag_name = self.tag_name(version)
         if self._has_version(self.branch, tag_name):
             return self.branch.tags.lookup_tag(tag_name)
-        debian_tag_name = "debian-" + tag_name
-        if self._has_version(self.branch, debian_tag_name):
-            return self.branch.tags.lookup_tag(debian_tag_name)
-        ubuntu_tag_name = "ubuntu-" + tag_name
-        if self._has_version(self.branch, ubuntu_tag_name):
-            return self.branch.tags.lookup_tag(ubuntu_tag_name)
+        for debian_tag_name in ["debian-" + tag_name, "debian/" + tag_name]:
+            if self._has_version(self.branch, debian_tag_name):
+                return self.branch.tags.lookup_tag(debian_tag_name)
+        for ubuntu_tag_name in ["ubuntu-" + tag_name, "ubuntu/" + tag_name]:
+            if self._has_version(self.branch, ubuntu_tag_name):
+                return self.branch.tags.lookup_tag(ubuntu_tag_name)
         return self.branch.tags.lookup_tag(tag_name)
 
     def tag_version(self, version, revid=None):
@@ -548,7 +555,7 @@ class DistributionBranch(object):
                         up_branch.repository)
                 pristine_upstream_revids = branch.pristine_upstream_source.version_as_revisions(
                         package, version, tarballs=upstream_tarballs)
-                for (component, pristine_upstream_revid) in pristine_upstream_revids.iteritems():
+                for (component, pristine_upstream_revid) in viewitems(pristine_upstream_revids):
                     if not graph.is_ancestor(up_branch.last_revision(),
                             pristine_upstream_revid):
                         return False
@@ -650,7 +657,7 @@ class DistributionBranch(object):
         assert isinstance(version, str)
         pull_revisions = pull_branch.pristine_upstream_source.version_as_revisions(
             package, version)
-        for (component, pull_revision) in pull_revisions.iteritems():
+        for (component, pull_revision) in viewitems(pull_revisions):
             mutter("Fetching upstream part %s of %s from revision %s" % \
                     (component, version, pull_revision))
             assert self.pristine_upstream_tree is not None, \
@@ -792,7 +799,6 @@ class DistributionBranch(object):
         # from lesser branches first? For now we'll just dump it on.
         # TODO: this method needs a lot of work for when we will make
         # the branches writeable by others.
-        assert isinstance(version, str)
         mutter("Importing upstream version %s from %s with parents %r" \
                 % (version, upstream_part, upstream_parents))
         assert self.pristine_upstream_tree is not None, \
@@ -1037,7 +1043,7 @@ class DistributionBranch(object):
             if not pull_branch.is_version_native(pull_version):
                 pull_revids = pull_branch.pristine_upstream_source.version_as_revisions(
                     package, pull_version.upstream_version)
-                if pull_revids.keys() != [None]:
+                if list(pull_revids.keys()) != [None]:
                     raise MultipleUpstreamTarballsNotSupported()
                 pull_revid = pull_revids[None]
                 mutter("Initialising upstream from %s, version %s",
@@ -1053,7 +1059,8 @@ class DistributionBranch(object):
 
     def get_changelog_from_source(self, dir, max_blocks=None):
         cl_filename = os.path.join(dir, "debian", "changelog")
-        content = open(cl_filename).read()
+        with open(cl_filename, 'rb') as f:
+            content = f.read()
         # Older versions of python-debian were accepting various encodings in
         # the changelog. This is not true with 0.1.20ubuntu2 at least which
         # force an 'utf-8' encoding. This leads to failures when trying to
@@ -1198,16 +1205,17 @@ class DistributionBranch(object):
             author = None
             if use_time_from_changelog and len(cl._blocks) > 0:
                  raw_timestamp = cl.date
-                 import rfc822, time
-                 time_tuple = rfc822.parsedate_tz(raw_timestamp)
+                 import email.utils, time
+                 time_tuple = email.utils.parsedate_tz(raw_timestamp)
                  if time_tuple is not None:
                      timestamp = (calendar.timegm(
                                   time_tuple[:9]) - time_tuple[9],
                                   time_tuple[9])
                  author = safe_decode(cl.author)
             versions = self._get_safe_versions_from_changelog(cl)
-            assert not self.has_version(version), \
-                "Trying to import version %s again" % str(version)
+            if self.has_version(version):
+                raise AssertionError(
+                        "Trying to import version %s again" % str(version))
             #TODO: check that the versions list is correctly ordered,
             # as some methods assume that, and it's not clear what
             # should happen if it isn't.
@@ -1237,7 +1245,8 @@ class DistributionBranch(object):
         to_location = os.path.join(basedir, "upstream")
         # Use upstream_branch if it has been set, otherwise self.branch.
         source_branch = self.pristine_upstream_branch or self.branch
-        assert upstream_tips.keys() == [None]
+        assert list(upstream_tips.keys()) == [None], "Upstream tips: %r" % list(upstream_tips.keys())
+        # TODO(jelmer): Use colocated branches rather than creating a copy.
         dir_to = source_branch.controldir.sprout(to_location,
                 revision_id=upstream_tips[None],
                 accelerator_tree=self.tree)
@@ -1247,12 +1256,14 @@ class DistributionBranch(object):
             # Handle shared treeless repo's.
             self.pristine_upstream_tree = dir_to.create_workingtree()
         self.pristine_upstream_branch = self.pristine_upstream_tree.branch
+        self.pristine_upstream_branch.get_config_stack().set('branch.fetch_tags', True)
 
     def _create_empty_upstream_tree(self, basedir):
         to_location = os.path.join(basedir, "upstream")
         to_transport = get_transport(to_location)
         to_transport.ensure_base()
-        format = controldir.format_registry.make_controldir('default')
+        source_branch = self.pristine_upstream_branch or self.branch
+        format = source_branch.controldir._format
         try:
             existing_controldir = controldir.ControlDir.open_from_transport(
                     to_transport)
@@ -1276,8 +1287,8 @@ class DistributionBranch(object):
             tip = self.branch.basis_tree()
             with tip.lock_read():
                 root_id = tip.path2id('')
-        if root_id:
-            self.pristine_upstream_tree.set_root_id(root_id)
+        if root_id and self.pristine_upstream_tree.supports_setting_file_ids():
+                self.pristine_upstream_tree.set_root_id(root_id)
 
     def _extract_tarballs_to_tempdir(self, tarballs):
         tempdir = tempfile.mkdtemp()
@@ -1291,14 +1302,12 @@ class DistributionBranch(object):
             raise
 
     def _export_previous_upstream_tree(self, package, previous_version, tempdir):
-        assert isinstance(previous_version, str), \
-            "Should pass upstream version as str, not Version."
         try:
             upstream_tips = self.pristine_upstream_source.version_as_revisions(
                     package, previous_version)
         except PackageVersionNotPresent:
             raise BzrCommandError("Unable to find the tag for the "
-                    "previous upstream version, %s, in the branch: "
+                    "previous upstream version (%s) in the branch: "
                     "%s" % (
                 previous_version,
                 self.pristine_upstream_source.tag_name(previous_version)))
@@ -1313,11 +1322,6 @@ class DistributionBranch(object):
     def merge_upstream(self, tarball_filenames, package, version, previous_version,
             upstream_branch=None, upstream_revisions=None, merge_type=None,
             force=False):
-        assert isinstance(version, str), \
-            "Should pass version as str not %s" % str(type(version))
-        assert isinstance(previous_version, str) or previous_version is None, \
-            "Should pass previous_version as str not %s" % str(
-                    type(previous_version))
         tempdir = tempfile.mkdtemp(dir=os.path.join(self.tree.basedir, '..'))
         try:
             if previous_version is not None:
@@ -1437,7 +1441,7 @@ class OneZeroSourceExtractor(SourceExtractor):
                             os.path.join(osutils.dirname(self.dsc_path),
                                 part['name'])),
                             component_from_orig_tarball(part['name'], name, str(version.upstream_version)),
-                            part['md5sum']))
+                            str(part['md5sum'])))
                 elif part['name'].endswith(".diff.gz"):
                     self.unextracted_debian_md5 = part['md5sum']
 
@@ -1503,11 +1507,11 @@ class ThreeDotZeroQuiltSourceExtractor(SourceExtractor):
                                     part['name'])),
                     component_from_orig_tarball(part['name'], name,
                         str(version.upstream_version)),
-                    part['md5sum']))
+                    str(part['md5sum'])))
             elif (part['name'].endswith(".debian.tar.gz")
                     or part['name'].endswith(".debian.tar.bz2")
                     or part['name'].endswith(".debian.tar.xz")):
-                self.unextracted_debian_md5 = part['md5sum']
+                self.unextracted_debian_md5 = str(part['md5sum'])
         assert self.upstream_tarballs is not None, \
             "Can't handle non gz|bz2|xz tarballs yet"
         assert self.unextracted_debian_md5 is not None, \
