@@ -51,14 +51,14 @@ def _upstream_version_data(branch, revid):
     changelog, _ignore = find_changelog(tree, False)
     uver = changelog.version.upstream_version
     upstream_revids = db.pristine_upstream_source.version_as_revisions(None, uver)
-    if upstream_revids.keys() != [None]:
+    if list(upstream_revids.keys()) != [None]:
         raise MultipleUpstreamTarballsNotSupported()
     upstream_revid = upstream_revids[None]
     return (Version(uver), upstream_revid)
 
 
 def fix_ancestry_as_needed(tree, source, source_revid=None):
-    """Manipulate the merge target's ancestry to avoid upstream conflicts.
+    r"""Manipulate the merge target's ancestry to avoid upstream conflicts.
 
     Merging J->I given the following ancestry tree is likely to result in
     upstream merge conflicts:
