@@ -30,11 +30,7 @@ from __future__ import absolute_import
 
 import subprocess
 
-try:
-    from debian.changelog import Version
-except ImportError:
-    # Prior to 0.1.15 the debian module was called debian_bundle
-    from debian_bundle.changelog import Version
+from debian.changelog import Version
 
 from .errors import (
     DchError,
@@ -51,9 +47,6 @@ def package_version(upstream_version, distribution_name, epoch=None):
     :param distribution_name: Distribution the package is for
     :param epoch: Optional epoch
     """
-    assert isinstance(upstream_version, str), \
-        "upstream_version should be a str, not %s" % str(
-                type(upstream_version))
     if distribution_name == "ubuntu":
         ret = Version("%s-0ubuntu1" % upstream_version)
     else:
@@ -91,9 +84,6 @@ def changelog_add_new_version(tree, upstream_version, distribution_name,
     :param changelog: Changelog object
     :param package: Package name
     """
-    assert isinstance(upstream_version, str), \
-         "upstream_version should be a str, not %s" % str(
-                 type(upstream_version))
     entry_description = upstream_merge_changelog_line(upstream_version)
     if changelog is None:
         epoch = None
