@@ -29,7 +29,6 @@ from testtools.content_type import ContentType
 from testtools.content import Content
 
 from .... import (
-    memorytree,
     merge,
     tests,
     )
@@ -41,7 +40,7 @@ from ....tests.features import ExecutableFeature
 dpkg_mergechangelogs_feature = ExecutableFeature('dpkg-mergechangelogs')
 
 
-v_111_2 = """\
+v_111_2 = b"""\
 pseudo-prog (1.1.1-2) unstable; urgency=low
 
   * New upstream release.
@@ -52,7 +51,7 @@ pseudo-prog (1.1.1-2) unstable; urgency=low
 """.splitlines(True)
 
 
-v_111_2b = """\
+v_111_2b = b"""\
 pseudo-prog (1.1.1-2) unstable; urgency=low
 
   * New upstream release.
@@ -64,7 +63,7 @@ pseudo-prog (1.1.1-2) unstable; urgency=low
 """.splitlines(True)
 
 
-v_111_2c = """\
+v_111_2c = b"""\
 pseudo-prog (1.1.1-2) unstable; urgency=low
 
   * New upstream release.
@@ -77,7 +76,7 @@ pseudo-prog (1.1.1-2) unstable; urgency=low
 
 # Merge of 2b and 2c using 2 as the base (b adds a line, c adds a line and
 # deletes a line).
-v_111_2bc = """\
+v_111_2bc = b"""\
 pseudo-prog (1.1.1-2) unstable; urgency=low
 
   * New upstream release.
@@ -91,7 +90,7 @@ pseudo-prog (1.1.1-2) unstable; urgency=low
 
 # Merge of 2b and 2c using an empty base. (As calculated by
 # dpkg-mergechangelogs.)
-v_111_2bc_empty_base = """\
+v_111_2bc_empty_base = b"""\
 pseudo-prog (1.1.1-2) unstable; urgency=low
 
   * New upstream release.
@@ -107,7 +106,7 @@ pseudo-prog (1.1.1-2) unstable; urgency=low
 """.splitlines(True)
 
 
-v_112_1 = """\
+v_112_1 = b"""\
 pseudo-prog (1.1.2-1) unstable; urgency=low
 
   * New upstream release.
@@ -118,7 +117,7 @@ pseudo-prog (1.1.2-1) unstable; urgency=low
 """.splitlines(True)
 
 
-v_001_1 = """\
+v_001_1 = b"""\
 pseudo-prog (0.0.1-1) unstable; urgency=low
 
   * New project released!!!!
@@ -169,7 +168,7 @@ class TestMergeChangelog(tests.TestCase):
             self.assertEqual('conflicted', status)
         else:
             self.assertEqual('success', status)
-        self.assertEqualDiff(''.join(expected_lines), ''.join(merged_lines))
+        self.assertEqualDiff(b''.join(expected_lines), b''.join(merged_lines))
         return True
 
     def test_merge_by_version(self):
@@ -229,7 +228,7 @@ class TestMergeChangelog(tests.TestCase):
             conflicted=True)
 
     def test_not_valid_changelog(self):
-        invalid_changelog = """\
+        invalid_changelog = b"""\
 pseudo-prog (1.1.1-2) unstable; urgency=low
 
   * New upstream release.
@@ -250,9 +249,9 @@ pseudo-prog (1.1.1-2) unstable; urgency=low
         #  - Andrew Bennetts, 25 July 2011.
         #self.assertEqual(''.join(invalid_changelog), ''.join(lines))
         self.assertMergeChangelog(v_112_1 + 
-                                  ['<<<<<<<\n'] +
+                                  [b'<<<<<<<\n'] +
                                   v_111_2 +
-                                  ['=======\n>>>>>>>\n'],
+                                  [b'=======\n>>>>>>>\n'],
                                   this_lines=v_111_2,
                                   other_lines=v_112_1,
                                   base_lines=invalid_changelog,
@@ -267,7 +266,7 @@ pseudo-prog (1.1.1-2) unstable; urgency=low
         In that case, the result should not be a success with a zero byte
         merge result file. See lp:893495 for such an issue.
         """
-        invalid_changelog = """\
+        invalid_changelog = b"""\
 pseudo-prog (ss-0) unstable; urgency=low
 
   * New project released!!!!
@@ -283,16 +282,17 @@ pseudo-prog (ss-0) unstable; urgency=low
             possible_error=True)
         if not handled:
             # Can't assert on the exact message as it depends on the locale
-            self.assertContainsRe(self.logged_warnings.getvalue(),
+            self.assertContainsRe(
+                self.logged_warnings.getvalue(),
                 "dpkg-mergechangelogs: .*ss-0( is not a valid version)?")
 
     def test_invalid_version_non_ascii(self):
         """Invalid version with non-ascii data is rejected or correctly merged
-        
+
         Such a version has always been treated as invalid so fails
         consistently across dpkg versions currently.
         """
-        invalid_changelog = """\
+        invalid_changelog = b"""\
 pseudo-prog (\xc2\xa7) unstable; urgency=low
 
   * New project released!!!!
@@ -320,36 +320,38 @@ class TestChangelogHook(tests.TestCaseWithMemoryTransport):
         builder = self.make_branch_builder('source')
         builder.start_series()
         builder.build_snapshot(None, [
-            ('add', ('', 'TREE_ROOT', 'directory', None)),
-            ('add', ('debian', 'deb-id', 'directory', None)),
-            ('add', ('debian/changelog', 'c-id', 'file', '')),
-            ('add', ('changelog', 'o-id', 'file', '')),
+            ('add', ('', b'TREE_ROOT', 'directory', None)),
+            ('add', ('debian', b'deb-id', 'directory', None)),
+            ('add', ('debian/changelog', b'c-id', 'file', b'')),
+            ('add', ('changelog', b'o-id', 'file', b'')),
             ])
         builder.finish_series()
         the_branch = builder.get_branch()
 
-        tree = memorytree.MemoryTree.create_on_branch(the_branch)
+        tree = the_branch.create_memorytree()
         tree.lock_write()
         self.addCleanup(tree.unlock)
 
         class FakeMerger(object):
+
             def __init__(self, this_tree):
                 self.this_tree = this_tree
-            def get_lines(self, tree, path, file_id):
-                return tree.get_file_lines(path, file_id)
+
+            def get_lines(self, tree, path):
+                return tree.get_file_lines(path)
 
         merger = FakeMerger(tree)
         params_cls = merge.MergeFileHookParams
-        params = params_cls(merger, 'c-id', ('debian/changelog',
-            'debian/changelog', 'debian/changelog'), None, 'file', 'file',
-            'this')
+        params = params_cls(
+            merger, b'c-id',
+            ('debian/changelog', 'debian/changelog', 'debian/changelog'),
+            None, 'file', 'file', 'this')
         return params, merger
 
-
     def test_changelog_merge_hook_successful(self):
         params, merger = self.make_params()
-        params.other_lines = ['']
-        params.base_lines = ['']
+        params.other_lines = [b'']
+        params.base_lines = [b'']
         file_merger = debian.changelog_merge_hook_factory(merger)
         result, new_content = file_merger.merge_text(params)
         self.assertEqual('success', result)
