@@ -50,6 +50,7 @@ from . import (
     TestCaseWithTransport,
     )
 from ..util import (
+    changelog_find_previous_upload,
     component_from_orig_tarball,
     dget,
     dget_changes,
@@ -58,7 +59,7 @@ from ..util import (
     find_changelog,
     find_extra_authors,
     find_last_distribution,
-    _find_previous_upload,
+    find_previous_upload,
     find_thanks,
     get_commit_info_from_changelog,
     guess_build_type,
@@ -76,6 +77,7 @@ from ..util import (
     )
 
 from .... import errors as bzr_errors
+from ....sixish import text_type
 from ....tests import (
     TestCase,
     )
@@ -93,11 +95,8 @@ class RecursiveCopyTests(TestCaseInTempDir):
         os.mkdir('c')
         os.mkdir('a/d')
         os.mkdir('a/d/e')
-        f = open('a/f', 'wb')
-        try:
+        with open('a/f', 'w') as f:
             f.write('f')
-        finally:
-            f.close()
         os.mkdir('b/g')
         recursive_copy('a', 'b')
         self.assertPathExists('a')
@@ -118,10 +117,10 @@ class SafeDecodeTests(TestCase):
 
     def test_utf8(self):
         self.assertSafeDecode(u'ascii', 'ascii')
-        self.assertSafeDecode(u'\xe7', '\xc3\xa7')
+        self.assertSafeDecode(u'\xe7', b'\xc3\xa7')
 
     def test_iso_8859_1(self):
-        self.assertSafeDecode(u'\xe7', '\xe7')
+        self.assertSafeDecode(u'\xe7', b'\xe7')
 
 
 cl_block1 = """\
@@ -139,7 +138,7 @@ bzr-builddeb (0.17) unstable; urgency=low
 class FindChangelogTests(TestCaseWithTransport):
 
     def write_changelog(self, filename):
-        f = open(filename, 'wb')
+        f = open(filename, 'w')
         try:
             f.write(cl_block1)
             f.write("""\
@@ -220,11 +219,8 @@ bzr-builddeb (0.16.2) unstable; urgency=low
         tree = self.make_branch_and_tree('.')
         os.mkdir('debian')
         self.write_changelog('debian/changelog')
-        f = open('changelog', 'wb')
-        try:
+        with open('changelog', 'w') as f:
             f.write('Naughty, naughty')
-        finally:
-            f.close()
         tree.add(['changelog', 'debian', 'debian/changelog'])
         (cl, lq) = find_changelog(tree, True)
         self.assertEqual(str(cl), cl_block1)
@@ -355,11 +351,8 @@ class MoveFileTests(TestCaseInTempDir):
     def test_move_file_same_md5(self):
         self.build_tree(['a'])
         md5sum = md5.md5()
-        f = open('a', 'rb')
-        try:
+        with open('a', 'rb') as f:
             md5sum.update(f.read())
-        finally:
-            f.close()
         shutil.copy('a', 'b')
         move_file_if_different('a', 'b', md5sum.hexdigest())
         self.assertPathExists('a')
@@ -368,54 +361,45 @@ class MoveFileTests(TestCaseInTempDir):
     def test_move_file_diff_md5(self):
         self.build_tree(['a', 'b'])
         md5sum = md5.md5()
-        f = open('a', 'rb')
-        try:
+        with open('a', 'rb') as f:
             md5sum.update(f.read())
-        finally:
-            f.close()
         a_hexdigest = md5sum.hexdigest()
         md5sum = md5.md5()
-        f = open('b', 'rb')
-        try:
+        with open('b', 'rb') as f:
             md5sum.update(f.read())
-        finally:
-            f.close()
         b_hexdigest = md5sum.hexdigest()
         self.assertNotEqual(a_hexdigest, b_hexdigest)
         move_file_if_different('a', 'b', a_hexdigest)
         self.assertPathDoesNotExist('a')
         self.assertPathExists('b')
         md5sum = md5.md5()
-        f = open('b', 'rb')
-        try:
+        with open('b', 'rb') as f:
             md5sum.update(f.read())
-        finally:
-            f.close()
         self.assertEqual(md5sum.hexdigest(), a_hexdigest)
 
 
 class WriteFileTests(TestCaseInTempDir):
 
     def test_write_non_extant(self):
-        write_if_different("foo", 'a')
+        write_if_different(b"foo", 'a')
         self.assertPathExists('a')
-        self.check_file_contents('a', "foo")
+        self.check_file_contents('a', b"foo")
 
     def test_write_file_same(self):
-        write_if_different("foo", 'a')
+        write_if_different(b"foo", 'a')
         self.assertPathExists('a')
-        self.check_file_contents('a', "foo")
-        write_if_different("foo", 'a')
+        self.check_file_contents('a', b"foo")
+        write_if_different(b"foo", 'a')
         self.assertPathExists('a')
-        self.check_file_contents('a', "foo")
+        self.check_file_contents('a', b"foo")
 
     def test_write_file_different(self):
-        write_if_different("foo", 'a')
+        write_if_different(b"foo", 'a')
         self.assertPathExists('a')
-        self.check_file_contents('a', "foo")
-        write_if_different("bar", 'a')
+        self.check_file_contents('a', b"foo")
+        write_if_different(b"bar", 'a')
         self.assertPathExists('a')
-        self.check_file_contents('a', "bar")
+        self.check_file_contents('a', b"bar")
 
 
 class DgetTests(TestCaseWithTransport):
@@ -504,14 +488,14 @@ class ChangelogInfoTests(TestCaseWithTransport):
                    "  [ B. Hacker ]", "  [ A. Hacker}"]
         authors = find_extra_authors(changes)
         self.assertEqual([u"A. Hacker", u"B. Hacker"], authors)
-        self.assertEqual([unicode]*len(authors), map(type, authors))
+        self.assertEqual([text_type]*len(authors), list(map(type, authors)))
 
     def test_find_extra_authors_utf8(self):
-        changes = ["  * Do foo", "", "  [ \xc3\xa1. Hacker ]", "  * Do bar", "",
-                   "  [ \xc3\xa7. Hacker ]", "  [ A. Hacker}"]
+        changes = [u"  * Do foo", u"", "  [ \xe1. Hacker ]", "  * Do bar", "",
+                   u"  [ \xe7. Hacker ]", "  [ A. Hacker}"]
         authors = find_extra_authors(changes)
         self.assertEqual([u"\xe1. Hacker", u"\xe7. Hacker"], authors)
-        self.assertEqual([unicode]*len(authors), map(type, authors))
+        self.assertEqual([text_type]*len(authors), list(map(type, authors)))
 
     def test_find_extra_authors_iso_8859_1(self):
         # We try to treat lines as utf-8, but if that fails to decode, we fall
@@ -520,7 +504,7 @@ class ChangelogInfoTests(TestCaseWithTransport):
                    "  [ \xe7. Hacker ]", "  [ A. Hacker}"]
         authors = find_extra_authors(changes)
         self.assertEqual([u"\xe1. Hacker", u"\xe7. Hacker"], authors)
-        self.assertEqual([unicode]*len(authors), map(type, authors))
+        self.assertEqual([text_type]*len(authors), list(map(type, authors)))
 
     def test_find_extra_authors_no_changes(self):
         authors = find_extra_authors([])
@@ -529,7 +513,7 @@ class ChangelogInfoTests(TestCaseWithTransport):
     def assert_thanks_is(self, changes, expected_thanks):
         thanks = find_thanks(changes)
         self.assertEqual(expected_thanks, thanks)
-        self.assertEqual([unicode]*len(thanks), map(type, thanks))
+        self.assertEqual([text_type]*len(thanks), list(map(type, thanks)))
 
     def test_find_thanks_no_changes(self):
         self.assert_thanks_is([], [])
@@ -557,9 +541,9 @@ class ChangelogInfoTests(TestCaseWithTransport):
         self.assert_thanks_is(changes, [u"Mark A. Super-Hacker"])
         changes = ["  * Thanks to A. Hacker <ahacker@example.com>"]
         self.assert_thanks_is(changes, [u"A. Hacker <ahacker@example.com>"])
-        changes = ["  * Thanks to Adeodato Sim\xc3\x83\xc2\xb3"]
+        changes = [u"  * Thanks to Adeodato Sim\xc3\xb3"]
         self.assert_thanks_is(changes, [u"Adeodato Sim\xc3\xb3"])
-        changes = ["  * Thanks to \xc3\x81deodato Sim\xc3\x83\xc2\xb3"]
+        changes = [u"  * Thanks to \xc1deodato Sim\xc3\xb3"]
         self.assert_thanks_is(changes, [u"\xc1deodato Sim\xc3\xb3"])
 
     def test_find_bugs_fixed_no_changes(self):
@@ -634,7 +618,7 @@ class ChangelogInfoTests(TestCaseWithTransport):
                         _lplib=MockLaunchpad())
         self.assertEqual("\n".join(strip_changelog_message(changes)), message)
         self.assertEqual([author]+find_extra_authors(changes), authors)
-        self.assertEqual(unicode, type(authors[0]))
+        self.assertEqual(text_type, type(authors[0]))
         self.assertEqual(find_thanks(changes), thanks)
         self.assertEqual(find_bugs_fixed(changes, wt.branch,
                     _lplib=MockLaunchpad()), bugs)
@@ -652,22 +636,14 @@ class ChangelogInfoTests(TestCaseWithTransport):
                          u'* Second change, thanks to \xde. Hacker',
                          message)
         self.assertEqual([author, u'\xc1. Hacker'], authors)
-        self.assertEqual(unicode, type(authors[0]))
+        self.assertEqual(text_type, type(authors[0]))
         self.assertEqual([u'\xde. Hacker'], thanks)
         self.assertEqual(['https://launchpad.net/bugs/12345 fixed'], bugs)
 
-    def test_get_commit_info_utf8(self):
-        changes = ["  [ \xc3\x81. Hacker ]",
-                   "  * First ch\xc3\xa1nge, LP: #12345",
-                   "  * Second change, thanks to \xc3\x9e. Hacker"]
-        self.assertUnicodeCommitInfo(changes)
-
-    def test_get_commit_info_iso_8859_1(self):
-        # Changelogs aren't always well-formed UTF-8, so we fall back to
-        # iso-8859-1 if we fail to decode utf-8.
-        changes = ["  [ \xc1. Hacker ]",
-                   "  * First ch\xe1nge, LP: #12345",
-                   "  * Second change, thanks to \xde. Hacker"]
+    def test_get_commit_info_unicode(self):
+        changes = [u"  [ \xc1. Hacker ]",
+                   u"  * First ch\xe1nge, LP: #12345",
+                   u"  * Second change, thanks to \xde. Hacker"]
         self.assertUnicodeCommitInfo(changes)
 
 
@@ -738,41 +714,41 @@ class FindPreviousUploadTests(TestCase):
     def test_find_previous_upload_debian(self):
         cl = self.make_changelog([("0.1-1", "unstable"),
                 ("0.1-2", "unstable")])
-        self.assertEqual(Version("0.1-1"), _find_previous_upload(cl))
+        self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
         cl = self.make_changelog([("0.1-1", "unstable"),
                 ("0.1-1.1", "stable-security"), ("0.1-2", "unstable")])
-        self.assertEqual(Version("0.1-1"), _find_previous_upload(cl))
+        self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
 
     def test_find_previous_upload_ubuntu(self):
         cl = self.make_changelog([("0.1-1", "lucid"),
                 ("0.1-2", "lucid")])
-        self.assertEqual(Version("0.1-1"), _find_previous_upload(cl))
+        self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
         cl = self.make_changelog([("0.1-1", "lucid"),
                 ("0.1-1.1", "unstable"), ("0.1-2", "maverick")])
         self.requireFeature(DistroInfoFeature)
-        self.assertEqual(Version("0.1-1"), _find_previous_upload(cl))
+        self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
 
     def test_find_previous_upload_ubuntu_pocket(self):
         cl = self.make_changelog([("0.1-1", "lucid-updates"),
                 ("0.1-2", "lucid-updates")])
-        self.assertEqual(Version("0.1-1"), _find_previous_upload(cl))
+        self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
 
     def test_find_previous_upload_unknown(self):
         cl = self.make_changelog([("0.1-1", "lucid"),
                 ("0.1-2", "dunno")])
-        self.assertRaises(NoPreviousUpload, _find_previous_upload, cl)
+        self.assertRaises(NoPreviousUpload, changelog_find_previous_upload, cl)
 
     def test_find_previous_upload_missing(self):
         cl = self.make_changelog([("0.1-1", "unstable"),
                 ("0.1-2", "lucid")])
-        self.assertRaises(NoPreviousUpload, _find_previous_upload, cl)
+        self.assertRaises(NoPreviousUpload, changelog_find_previous_upload, cl)
         cl = self.make_changelog([("0.1-1", "unstable")])
-        self.assertRaises(NoPreviousUpload, _find_previous_upload, cl)
+        self.assertRaises(NoPreviousUpload, changelog_find_previous_upload, cl)
 
     def test_find_previous_upload_unreleased(self):
         cl = self.make_changelog([("0.1-1", "unstable"),
                 ("0.1-2", "UNRELEASED")])
-        self.assertEqual(Version("0.1-1"), _find_previous_upload(cl))
+        self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
 
 
 class SourceFormatTests(TestCaseWithTransport):
