@@ -32,8 +32,6 @@ import tempfile
 import os
 import re
 
-from ...trace import mutter
-
 from debian import deb822
 from debian.changelog import Changelog, ChangelogParseError
 
@@ -43,7 +41,11 @@ from ... import (
     urlutils,
     version_info as bzr_version_info,
     )
-from ...trace import warning
+from ...sixish import text_type
+from ...trace import (
+    mutter,
+    warning,
+    )
 from ...transport import (
     do_catching_redirections,
     get_transport,
@@ -107,12 +109,12 @@ UBUNTU_POCKETS = ('', '-proposed', '-updates', '-security', '-backports')
 
 def safe_decode(s):
     """Decode a string into a Unicode value."""
-    if isinstance(s, unicode): # Already unicode
-        mutter('safe_decode() called on an already-unicode string: %r' % (s,))
+    if isinstance(s, text_type): # Already unicode
+        mutter('safe_decode() called on an already-decoded string: %r' % (s,))
         return s
     try:
         return s.decode('utf-8')
-    except UnicodeDecodeError, e:
+    except UnicodeDecodeError as e:
         mutter('safe_decode(%r) falling back to iso-8859-1' % (s,))
         # TODO: Looking at BeautifulSoup it seems to use 'chardet' to try to
         #       guess the encoding of a given text stream. We might want to
@@ -141,7 +143,7 @@ def recursive_copy(fromdir, todir):
             shutil.copy(path, todir)
 
 
-def find_changelog(t, merge, max_blocks=1):
+def find_changelog(t, merge=False, max_blocks=1):
     """Find the changelog in the given tree.
 
     First looks for 'debian/changelog'. If "merge" is true will also
@@ -199,7 +201,7 @@ def find_changelog(t, merge, max_blocks=1):
     changelog = Changelog()
     try:
         changelog.parse_changelog(contents, max_blocks=max_blocks, allow_empty_author=True)
-    except ChangelogParseError, e:
+    except ChangelogParseError as e:
         raise UnparseableChangelog(str(e))
     return changelog, top_level
 
@@ -223,12 +225,12 @@ def strip_changelog_message(changes):
         changes.pop(0)
 
     whitespace_column_re = re.compile(r'  |\t')
-    changes = map(lambda line: whitespace_column_re.sub('', line, 1), changes)
+    changes = [whitespace_column_re.sub('', line, 1) for line in changes]
 
     leader_re = re.compile(r'[ \t]*[*+-] ')
-    count = len(filter(leader_re.match, changes))
+    count = len([l for l in changes if leader_re.match(l)])
     if count == 1:
-        return map(lambda line: leader_re.sub('', line, 1).lstrip(), changes)
+        return [leader_re.sub('', line, 1).lstrip() for line in changes]
     else:
         return changes
 
@@ -324,7 +326,7 @@ def write_if_different(contents, target):
     md5sum = md5.md5()
     md5sum.update(contents)
     fd, temp_path = tempfile.mkstemp("builddeb-rename-")
-    fobj = os.fdopen(fd, "wd")
+    fobj = os.fdopen(fd, "wb")
     try:
         try:
             fobj.write(contents)
@@ -390,6 +392,15 @@ def open_file_via_transport(filename, transport):
 
 
 def _dget(cls, dsc_location, target_dir):
+    """Copy all files referenced by a .dsc file.
+
+    Args:
+      cls: Parser class
+      dsc_location: Source file location
+      target_dir: Target directory
+    Return:
+      path to target source file
+    """
     if not os.path.isdir(target_dir):
         raise errors.NotADirectory(target_dir)
     path, dsc_t = open_transport(dsc_location)
@@ -430,21 +441,21 @@ def find_bugs_fixed(changes, branch, _lplib=None):
         from . import launchpad as _lplib
     bugs = []
     for change in changes:
-        for match in re.finditer("closes:\s*(?:bug)?\#?\s?\d+"
-                "(?:,\s*(?:bug)?\#?\s?\d+)*", change,
+        for match in re.finditer("closes:\\s*(?:bug)?\\#?\\s?\\d+"
+                "(?:,\\s*(?:bug)?\\#?\\s?\\d+)*", change,
                 re.IGNORECASE):
             closes_list = match.group(0)
-            for match in re.finditer("\d+", closes_list):
+            for match in re.finditer("\\d+", closes_list):
                 bug_url = bugtracker.get_bug_url("deb", branch, match.group(0))
                 bugs.append(bug_url + " fixed")
                 lp_bugs = _lplib.ubuntu_bugs_for_debian_bug(match.group(0))
                 if len(lp_bugs) == 1:
                     bug_url = bugtracker.get_bug_url("lp", branch, lp_bugs[0])
                     bugs.append(bug_url + " fixed")
-        for match in re.finditer("lp:\s+\#\d+(?:,\s*\#\d+)*",
+        for match in re.finditer("lp:\\s+\\#\\d+(?:,\\s*\\#\\d+)*",
                 change, re.IGNORECASE):
             closes_list = match.group(0)
-            for match in re.finditer("\d+", closes_list):
+            for match in re.finditer("\\d+", closes_list):
                 bug_url = bugtracker.get_bug_url("lp", branch, match.group(0))
                 bugs.append(bug_url + " fixed")
                 deb_bugs = _lplib.debian_bugs_for_ubuntu_bug(match.group(0))
@@ -483,8 +494,8 @@ def find_thanks(changes):
     :return: List of people thanked, optionally including email address.
     """
     thanks_re = re.compile(r"[tT]hank(?:(?:s)|(?:you))(?:\s*to)?"
-            "((?:\s+(?:(?:\w\.)|(?:\w+(?:-\w+)*)))+"
-            "(?:\s+<[^@>]+@[^@>]+>)?)",
+            "((?:\\s+(?:(?:\\w\\.)|(?:\\w+(?:-\\w+)*)))+"
+            "(?:\\s+<[^@>]+@[^@>]+>)?)",
             re.UNICODE)
     thanks = []
     changes_str = safe_decode(" ".join(changes))
@@ -583,7 +594,7 @@ def debuild_config(tree, working_tree):
     return config
 
 
-def find_previous_upload(tree, merge):
+def find_previous_upload(tree, merge=False):
     """Given a tree, find the previous upload to the distribution.
 
     When e.g. Ubuntu merges from Debian they want to build with
@@ -604,10 +615,10 @@ def find_previous_upload(tree, merge):
         cl, top_level = find_changelog(tree, merge, max_blocks=None)
     except UnparseableChangelog:
         raise UnableToFindPreviousUpload()
-    return _find_previous_upload(cl)
+    return changelog_find_previous_upload(cl)
 
 
-def _find_previous_upload(cl):
+def changelog_find_previous_upload(cl):
     """Find the version of the previous upload.
 
     :param cl: Changelog object
@@ -662,13 +673,13 @@ def tree_get_source_format(tree):
     filename = "debian/source/format"
     try:
         text = tree.get_file_text(filename)
-    except IOError as (num, unused_msg):
-        if num == errno.ENOENT:
+    except IOError as e:
+        if e.errno == errno.ENOENT:
             return FORMAT_1_0
         raise
     except errors.NoSuchFile:
         return FORMAT_1_0
-    return text.strip()
+    return text.strip().decode('ascii')
 
 
 FORMAT_1_0 = "1.0"
@@ -778,3 +789,14 @@ def extract_orig_tarballs(tarballs, target, strip_components=None):
     for tarball_filename, component in tarballs:
         extract_orig_tarball(tarball_filename, component, target,
             strip_components=strip_components)
+
+
+def dput_changes(path):
+    """Upload a package."""
+    (bd, changes_file) = os.path.split(path)
+    subprocess.check_call(["dput", changes_file], cwd=bd)
+
+
+def debsign(path):
+    (bd, changes_file) = os.path.split(path)
+    subprocess.check_call(["debsign", changes_file], cwd=bd)
